/***
 * Created by Bryan Tai and Carolyn Shen Mar 25, 2016
 * Created for Project 4 of CPSC 314
 *
*/
var view = (function(V){
  "use strict";
  var canvas = document.getElementById('canvas');
  var scene = V.scene;  
  
  var textures = {
	"crosshairTexture":THREE.ImageUtils.loadTexture("images/crosshairs.png"),
    "ceilingTexture" : repeatTexture(THREE.ImageUtils.loadTexture("images/ceiling.jpg"), 4, 4),
    "doorTexture" : THREE.ImageUtils.loadTexture("images/wood.jpg"),
    "wallTexture" : repeatTexture(THREE.ImageUtils.loadTexture("images/wall.jpg"), 2,2),
	"blueWallTexture" : THREE.ImageUtils.loadTexture("images/bluewall.jpg"),
    "tableTexture" : THREE.ImageUtils.loadTexture("images/darkwood.jpg"),
    "floorTexture" : repeatTexture(THREE.ImageUtils.loadTexture("images/carpet.jpg"),2,2),
    "cabinetTexture" : {color:0xffffff},
    "fridgeTexture" : {color:0xffffff},
    "ovenTexture" : {color:0xffffff},
    "couchTexture" : THREE.ImageUtils.loadTexture("images/couch.jpg"),
    "switchTexture" : {color:0xffffff},
    "lockerTexture" : {color:0xffffff},
    "shelfTexture" : THREE.ImageUtils.loadTexture("images/wood.jpg"),
    "computerTexture" : THREE.ImageUtils.loadTexture("images/plastic.jpg"),
    "whiteboardTexture" : THREE.ImageUtils.loadTexture("images/whiteboard.jpg"),
    "shinyBlackTexture": THREE.ImageUtils.loadTexture("images/stock.jpg"),
	"bsodTexture": THREE.ImageUtils.loadTexture("images/bsod.jpg"),
	"loginTexture": THREE.ImageUtils.loadTexture("images/login.jpg"),
	"loggedOnTexture":THREE.ImageUtils.loadTexture("images/loggedOn.jpg"),
	"tvTexture": THREE.ImageUtils.loadTexture("images/ssb.png"),
	"bluePlasticTexture": THREE.ImageUtils.loadTexture("images/bluePlastic.jpg"),
	"greyPlasticTexture": THREE.ImageUtils.loadTexture("images/greyPlastic.jpg"),
	"orangePlasticTexture": THREE.ImageUtils.loadTexture("images/orangePlastic.jpg"),
	"bulletBoxTexture": THREE.ImageUtils.loadTexture("images/nerf.png"),
	"bulletTexture": THREE.ImageUtils.loadTexture("images/bsod.jpg"),
	"cardboardTexture": THREE.ImageUtils.loadTexture("images/cardboard.jpg"),
	"keycardTexture": THREE.ImageUtils.loadTexture("images/bsod.jpg"),
	"metalTexture": THREE.ImageUtils.loadTexture("images/metal.jpg")
  };
  var renderer = new THREE.WebGLRenderer();
  renderer.setSize( window.innerWidth, window.innerHeight);
  var fpCamera = firstPersonCamera;

  var objectPicker = picker;
  var pickable = [];
  
  var bullet;
  var cameraGun;
  var highBox;
  var ceilingLights = [];
  var centerLight;
  //This is how the view knows which panel to open for a hollow box
  
  /*** We set up the renderer, camera and resizing using Project 2 code ***/
  //SETUP RENDERER & SCENE
  function initRenderer() {
    renderer.shadowMap.enabled = true; // enables shadow casting
    renderer.setClearColor(0xAEEEEE); // white background colour
    canvas.appendChild(renderer.domElement);
  }

  // ADAPT TO WINDOW RESIZE
  // Borrowed from P1
  function initResize(){
    function resize() {
        renderer.setSize(window.innerWidth,window.innerHeight);
        fpCamera.camera.aspect = window.innerWidth/window.innerHeight;
        fpCamera.camera.updateProjectionMatrix();
    }
    // EVENT LISTENER RESIZE
    window.addEventListener('resize',resize);
    resize();
  }

  function repeatTexture(texture, x, y) {
    texture.wrapS = texture.wrapT = THREE.RepeatWrapping;
    texture.repeat.set(x,y);
    return texture;
  }

  
  // LIGHTING UNIFORMS
  var lightColor = new THREE.Color(1,1,1); // I_l
  var ambientColor = new THREE.Color(0.4,0.4,0.4); // I_a
  var lightPosition = new THREE.Vector3(0,0,0);

  var kAmbient = 0.4; 
  var kDiffuse = 0.8;
  var kSpecular = 0.8;

  var specularExponent = 10;
  var roughness = 0.1;
  var coolMaterialColor = new THREE.Color(0.4,0.4,0.8); 
  var warmMaterialColor = new THREE.Color(0.8,0.4,0.4);
  var outlineColor = new THREE.Color(0.04,0.1,0.15); // TODO for TOON SHADING probably

  // SHADER MATERIALS
  var blackMetalMaterial = new THREE.ShaderMaterial({
     uniforms: {
		 lightPosition: {type: 'v3', value: lightPosition}
	 },
  });


 //LOAD SHADERS
var shaderFiles = [
  'glsl/metal.vs.glsl',
  'glsl/metal.fs.glsl'
];
  
new THREE.SourceLoader().load(shaderFiles, function(shaders) {
  blackMetalMaterial.vertexShader = shaders['glsl/metal.vs.glsl'];
  blackMetalMaterial.fragmentShader = shaders['glsl/metal.fs.glsl'];
  blackMetalMaterial.needsUpdate = true;
})

  function initLockMouse() {
    document.addEventListener('click', function(e) {
      var container = document.body;
      container.requestPointerLock = container.mozRequestPointerLock;
      container.requestPointerLock();
    }, false);

  }
  
  function initObjects() {
	  //TODO replace basic materials with Lambert materials for LIGHTS	
	var crosshairMaterial = new THREE.MeshLambertMaterial( {map: textures.crosshairTexture})
	crosshairMaterial.transparent = true;
	var floorMaterial = new THREE.MeshLambertMaterial( {map: textures.floorTexture});
	var ceilingMaterial = new THREE.MeshLambertMaterial( {map: textures.ceilingTexture} );
	var doorMaterial = new THREE.MeshLambertMaterial( {map: textures.doorTexture} );
	var wallMaterial = new THREE.MeshLambertMaterial( {map: textures.wallTexture} ) ;
	var blueWallMaterial = new THREE.MeshLambertMaterial( {map: textures.blueWallTexture} ) ;
	var tableMaterial = new THREE.MeshLambertMaterial( {map: textures.tableTexture} );
	var shelfMaterial = new THREE.MeshLambertMaterial( {map: textures.shelfTexture} );
	var couchMaterial = new THREE.MeshLambertMaterial( {map: textures.couchTexture});
	var whiteboardMaterial = new THREE.MeshLambertMaterial( {map: textures.whiteboardTexture});
	var computerMaterial = new THREE.MeshLambertMaterial( {map: textures.computerTexture});	
	var shinyBlackMaterial = new THREE.MeshLambertMaterial({map: textures.shinyBlackTexture});
	var bluePlasticMaterial = new THREE.MeshLambertMaterial({map: textures.bluePlasticTexture});
	var greyPlasticMaterial = new THREE.MeshLambertMaterial({map: textures.greyPlasticTexture});
	var orangePlasticMaterial = new THREE.MeshLambertMaterial({map: textures.orangePlasticTexture});
	var bulletBoxMaterial = new THREE.MeshLambertMaterial({map: textures.bulletBoxTexture});
	var bulletMaterial = new THREE.MeshLambertMaterial({map: textures.bulletTexture});
	var cardboardMaterial = new THREE.MeshLambertMaterial({map: textures.cardboardTexture});
	var keycardMaterial = new THREE.MeshLambertMaterial({map: textures.keycardTexture});
	var metalMaterial = new THREE.MeshLambertMaterial({map: textures.metalTexture});

	//TODO These are placeholder materials until we can find a nice texture or shader
	var cabinetMaterial = new THREE.MeshLambertMaterial({color:0xffffff});
	var fridgeMaterial = new THREE.MeshLambertMaterial({color:0xffffff} );
	var ovenMaterial = new THREE.MeshLambertMaterial( { color:0xffffff});
	var switchMaterial = new THREE.MeshLambertMaterial( {color:0xffffff});
	//TODO make this glow and pass in custom texture when monitor is created
	var screenMaterial = new THREE.MeshLambertMaterial( {color:0xffffff});
	//TODO make this reflective or just shiny
	var windowMaterial = new THREE.MeshLambertMaterial( {color:0xffffff});
	var lampMaterial = new THREE.MeshLambertMaterial( {color:0xffffff});
	//var lampMaterial = new THREE.MeshPhongMaterial( {color:0xffffff});
    
	//TODO perhaps we could create objects to hold all this info like we did with the celestialBodies in P2?
    
	var tableWidth = 80;
	var tableHeight = 70;
	var tableLength = 150;
	
	function makeCrosshairs(){
		var crossGeom = new THREE.PlaneGeometry(0.5,0.5);
		var crossMesh = new THREE.Mesh(crossGeom, crosshairMaterial);
		V.camera.add(crossMesh);
		crossMesh.translateZ(-2);
	}
	
    function makeFloor() {
		var floorGeometry = new THREE.BoxGeometry(V.ROOM_WIDTH,0.5,V.ROOM_LENGTH);
		var floorMesh = new THREE.Mesh(floorGeometry, floorMaterial);
        floorMesh.receiveShadow = true;
		scene.add(floorMesh);
	}
	
	function makeCeiling() {
      var ceilingGeometry = new THREE.BoxGeometry(V.ROOM_WIDTH,0.5,V.ROOM_LENGTH);
      var ceilingMesh = new THREE.Mesh(ceilingGeometry, ceilingMaterial);
		ceilingMesh.translateY(V.ROOM_HEIGHT);
		scene.add(ceilingMesh);
	}

	function makeWalls(){
		for(var i = 0; i < 4; i++){
			var material = (i < 2) ? blueWallMaterial : wallMaterial;
			var wall = makeSimpleBoxMesh(new THREE.Vector3(V.ROOM_LENGTH,V.ROOM_HEIGHT,0.5),"wall",material);
			wall.translateY(V.ROOM_HEIGHT/2);
			wall.rotateOnAxis(V.Y_AXIS, i * -Math.PI / 2);
			wall.translateZ(V.ROOM_WIDTH/2);
			wall.name = "wall" + i.toString();
			scene.add(wall);
		}
	}
	
	function makeSimpleBoxMesh(boxDimensions,boxName,boxMaterial){
		var boxGeom = new THREE.BoxGeometry(boxDimensions.x, boxDimensions.y, boxDimensions.z);
		var boxMesh = new THREE.Mesh(boxGeom, boxMaterial.clone());
		boxMesh.name = boxName;
		return boxMesh;
	}
	
	function makeRectangleMesh(rectWidth, rectHeight, rectName, rectImage){
		var rectDepth = 3;
		var rectDimensions = new THREE.Vector3(rectWidth,rectHeight,rectDepth);
		var rectMesh = makeSimpleBoxMesh(rectDimensions,rectName,rectImage);
		return rectMesh;
	}
	
	function makeBlocker(){
		var blockerDimensions = new THREE.Vector3(V.BLOCKER_WIDTH, V.ROOM_HEIGHT, V.BLOCKER_LENGTH);
		var blockerMesh = makeSimpleBoxMesh(blockerDimensions, "blocker", blueWallMaterial);
		
		blockerMesh.translateY(V.ROOM_HEIGHT/2);
		blockerMesh.translateX((V.ROOM_WIDTH- V.BLOCKER_WIDTH)/2);
		blockerMesh.translateZ((V.ROOM_LENGTH- V.BLOCKER_LENGTH)/2);
		scene.add(blockerMesh);
	}
	//TODO use simple box here
	function makeDoor(){
		var doorWidth = 90;
		var doorHeight = 210;
		var doorGeometry = new THREE.BoxGeometry(2, doorHeight, doorWidth);
		var doorMesh = new THREE.Mesh(doorGeometry, doorMaterial);
		doorMesh.translateY(doorHeight/2);
		doorMesh.translateX(V.ROOM_WIDTH/2);
		doorMesh.translateZ((-V.ROOM_LENGTH/2)+130+(doorWidth/2));
		doorMesh.name = "frontDoor";
		pickable.push(doorMesh);
		scene.add(doorMesh);
	}
	
	function makeLights(){
		var ambientLight = new THREE.AmbientLight( 0x222222 );
		scene.add( ambientLight );
		
		var lampWidth = 100;
		var lampDepth = 15;
		for(var a = 0; a < 4; a++){
			var lampMesh = makeRectangleMesh(lampWidth,lampDepth,"ceilingLight",lampMaterial);
			lampMesh.rotateOnAxis(V.X_AXIS, -Math.PI/2);
			lampMesh.translateZ(V.ROOM_HEIGHT);
			var yOffset = (a % 2 == 0) ? 100 : -100;
			var xOffset = (a < 2) ? 100 : -100;
			lampMesh.translateX(xOffset);
			lampMesh.translateY(yOffset);
			scene.add(lampMesh);
          
            var ceilingLight = new THREE.DirectionalLight(0xffffff, V.CEILING_LIGHT_INTENSITY, 0, 0.3);
            ceilingLight.castShadow = true;
            ceilingLight.shadowDarkness = 0.9;
			ceilingLight.rotateOnAxis(V.X_AXIS, -Math.PI/2);
            ceilingLight.translateZ(V.ROOM_HEIGHT);
            ceilingLight.translateX(xOffset);
            ceilingLight.translateY(yOffset);
            scene.add(ceilingLight);
            ceilingLights[a] = ceilingLight;
		}
        centerLight = new THREE.PointLight(0xffffff, V.CENTER_LIGHT_INTENSITY, 0, 0.3);
        centerLight.castShadow = false;
        centerLight.translateY(200);
        scene.add(centerLight);
	}
	
	function makeLightSwitch(){
		var lightSwitchMesh = makeSimpleBoxMesh(new THREE.Vector3(5,20,10),"lightSwitch",switchMaterial);
		
		lightSwitchMesh.translateY(110); //Half the door height
		lightSwitchMesh.translateX(V.ROOM_WIDTH/2);
		lightSwitchMesh.translateZ(-V.ROOM_LENGTH/2+100);

		pickable.push(lightSwitchMesh);
		scene.add(lightSwitchMesh);
	}
	
	//Use this for fridge, oven, locker, cabinets, etc
	function makeHollowBoxWithDoor(boxDimensions,boxThickness,boxName, boxMaterial){
		var thickness = boxThickness;
		var boxWidth = boxDimensions.x;
		var boxHeight = boxDimensions.y;
		var boxLength = boxDimensions.z;
		var panelWidth = boxWidth-thickness;
		var panelHeight = boxHeight-thickness;
		var panelLength = boxLength-thickness;
		
		var box = new THREE.Object3D();
		//box.name = boxName;
		for(var a = 0; a < 4; a++){
			var zSide = (a%2==0);
			var panelSide = (zSide) ? panelWidth : panelLength; 
			var panelOtherSide = (zSide) ? panelLength : panelWidth;

			var panelGeom = new THREE.BoxGeometry(panelSide, panelHeight, thickness);
			var panel = new THREE.Mesh(panelGeom, boxMaterial.clone());
			box.add(panel);
			panel.rotateOnAxis(V.Y_AXIS, a * Math.PI/2);
			panel.translateX(-thickness/2);
			panel.translateZ(panelOtherSide/2);
			
			//designate one panel to be a pickable door
			if(a == 0){
				panel.name = boxName;
				panel["doorIsOpen"] = false;
				pickable.push(panel);
			}
		}
		for(var b = 0; b < 2; b++){
			var coverGeom = new THREE.BoxGeometry(boxWidth, thickness, boxLength);
			var cover = new THREE.Mesh(coverGeom, boxMaterial.clone());
			box.add(cover);
			if(b ==0)
				cover.translateY((panelHeight+thickness)/2);
			else
				cover.translateY((-panelHeight-thickness)/2);
		}
		box.translateY(boxHeight/2);
		return box;
	}
	
	function makeCabinets(){
		var cabinetWidth = 120;
		var cabinetHeight = 90;
		var cabinetLength = 60;
		var cabinetDimensions = new THREE.Vector3(cabinetWidth, cabinetHeight, cabinetLength);
		
		var cabinet = makeHollowBoxWithDoor(cabinetDimensions, 5, "cabinet", cabinetMaterial);
		cabinet.translateY(V.ROOM_HEIGHT/2 );
		cabinet.translateX((V.ROOM_WIDTH- cabinetWidth)/2);
		cabinet.translateZ((cabinetLength-V.ROOM_LENGTH)/2);
		scene.add(cabinet);
		
		var sinkBase = makeHollowBoxWithDoor(cabinetDimensions, 5, "sinkBase", cabinetMaterial);
		sinkBase.translateX((V.ROOM_WIDTH- cabinetWidth)/2);
		sinkBase.translateZ((cabinetLength-V.ROOM_LENGTH)/2);
		scene.add(sinkBase);
	}
	
	function makeFridges(){
		var fridgeWidth = 60;
		var fridgeHeight = 150;
		var fridgeDepth = 70;
		var doorThickness = 5;
		
		function makeBottomMesh(name, bottomMaterial){
			var bottomDimensions = new THREE.Vector3(fridgeWidth, fridgeHeight*(2/3), fridgeDepth);
			var bottom = makeHollowBoxWithDoor(bottomDimensions, doorThickness, name, bottomMaterial);
			return bottom;
		}
		
		function makeFridgeMesh(){
			var freezerDimensions = new THREE.Vector3(fridgeWidth, fridgeHeight/3, fridgeDepth);
			
			var fridge = new THREE.Object3D();
			fridge.name = "fridge";
			
			var freezer = makeHollowBoxWithDoor(freezerDimensions, doorThickness, "freezer", fridgeMaterial);
			
			var bottom = makeBottomMesh("bottom",fridgeMaterial);
			
			fridge.add(freezer);
			fridge.add(bottom);
			freezer.translateY(fridgeHeight*(2/3));
			return fridge;
		}
		var bigFridge = makeFridgeMesh();
		bigFridge.translateX((V.ROOM_WIDTH/2)-155);
		bigFridge.translateZ((fridgeDepth-V.ROOM_LENGTH )/2);
		scene.add(bigFridge);
		
		var miniFridges = new THREE.Object3D();
		
		var metalMiniFridge = makeBottomMesh("metalFridge",metalMaterial); 
		metalMiniFridge.scale.set(0.9,0.9,0.9);
		var bulletBox = makeSimpleBoxMesh(new THREE.Vector3(30,20,20),"bulletBox",bulletBoxMaterial);
		bulletBox.translateY(-0.9*fridgeHeight/3);
		metalMiniFridge.add(bulletBox);
		pickable.push(bulletBox);
		
		miniFridges.add(metalMiniFridge);
		metalMiniFridge.translateY(2*fridgeHeight/3);
		
		var execMiniFridge = makeBottomMesh("execFridge",fridgeMaterial);
		miniFridges.add(execMiniFridge);
		
		miniFridges.translateX(100);
		miniFridges.translateZ((fridgeDepth-V.ROOM_LENGTH )/2);
		miniFridges.scale.set(0.8,0.8,0.8);
		scene.add(miniFridges);
	}

	function makeMicrowaveOvenMesh(ovenName){
		//TODO get measurements
		var ovenThickness = 5;
		var ovenDimensions = new THREE.Vector3(50, 25, 35);
		
		var oven = makeHollowBoxWithDoor(ovenDimensions, 5, ovenName, ovenMaterial);
		return oven;
	}
	
	function makeMonitorMesh(monitorName, monitorTexture){
		//TODO add pickable screen to the front
		var monitorWidth = 50;
		var monitorHeight = 40;
		var monitorDepth = 3;
		var baseWidth = 25;
		var baseHeight = 5;
		var baseDepth = 15;
		var supportWidth = 10;
		var supportHeight = 30;
		var supportDepth = 3;
		var screenOffset = 2;
		
		var monitor = new THREE.Object3D();
		
		var monitorGeom = new THREE.BoxGeometry(monitorWidth, monitorHeight, monitorDepth);
		var monitorMesh = new THREE.Mesh(monitorGeom, computerMaterial);
		monitor.add(monitorMesh);
		
		var baseGeom = new THREE.BoxGeometry(baseWidth, baseHeight, baseDepth);
		var baseMesh = new THREE.Mesh(baseGeom, computerMaterial);
		baseMesh.translateY(-monitorHeight*(4/5));
		monitor.add(baseMesh);
		
		var supportGeom = new THREE.BoxGeometry(supportWidth, supportHeight, supportDepth);
		var supportMesh = new THREE.Mesh(supportGeom, computerMaterial);
		supportMesh.translateZ(-(monitorDepth+supportDepth)/2);
		supportMesh.translateY(-monitorHeight/2);
		monitor.add(supportMesh);
		
		var screenGeom = new THREE.BoxGeometry(monitorWidth-screenOffset, monitorHeight-screenOffset, monitorDepth);
		var monitorMaterial = screenMaterial.clone();
		monitorMaterial.map = monitorTexture;
		var screenMesh = new THREE.Mesh(screenGeom, monitorMaterial);
		screenMesh.translateZ(screenOffset/4);
		screenMesh.name = monitorName;
		pickable.push(screenMesh);
		monitor.add(screenMesh);
		
		return monitor;
	}
	
	function makeTables(){
		function makeTableMesh(){
			var legWidth = 5;
			var tableTopGeometry = new THREE.BoxGeometry(tableWidth,3,tableLength);
			var tableTopMesh = new THREE.Mesh(tableTopGeometry, tableMaterial);
			tableTopMesh.translateY(tableHeight);
            tableTopMesh.castShadow = true;
			for(var i = 0; i < 4; i++){
				var tableLeg = new THREE.Mesh(new THREE.BoxGeometry(legWidth,tableHeight,legWidth), tableMaterial);
                if(i < 2)tableLeg.castShadow = true;
				var xOffset = (i < 2) ? (tableWidth/2)-legWidth : legWidth - (tableWidth/2);
				var zOffset = (i % 2 == 0) ? (tableLength/2)-legWidth : legWidth - (tableLength/2);
				tableTopMesh.add(tableLeg);
				tableLeg.translateY(-tableHeight/2);
				tableLeg.translateX(xOffset);
				tableLeg.translateZ(zOffset);
			}
			return tableTopMesh;
		}
		function makeComputerTable(){
			var computerTable = makeTableMesh();
			
			var computer = makeSimpleBoxMesh(new THREE.Vector3(20,45,45),"computer",computerMaterial);
            //computer.castShadow = true;
			computer.rotateOnAxis(V.Y_AXIS,-Math.PI/2);
			computer.translateX((tableLength/2) - 20);
			computer.translateY(-tableHeight/2-10);
			computerTable.add(computer);
			
			for(var m = 0; m<2;m++){
				//var texture = (m==0)? textures.bsodTexture:textures.loginTexture;
				var monitor = makeMonitorMesh("monitor"+m.toString(),textures.loginTexture);
				var xOffset = (m==0) ? tableWidth*(11/16) : 0;
				monitor.rotateOnAxis(V.Y_AXIS,-Math.PI/2);
				monitor.translateX(xOffset);
				monitor.translateY(tableHeight/2);
				monitor.translateZ(-tableWidth/4);
				computerTable.add(monitor);
			}
			computerTable.rotateOnAxis(V.Y_AXIS,Math.PI/2);
			computerTable.translateX((V.ROOM_LENGTH/2)-tableWidth/2);
			computerTable.translateZ((-V.ROOM_WIDTH/2)+tableWidth);
			scene.add(computerTable);
		}
		
		function makeMicrowaveTable(){
			var microwaveTable = makeTableMesh();
			var leftOven = makeMicrowaveOvenMesh("leftOven");
			leftOven.rotateOnAxis(V.Y_AXIS,-Math.PI/2);
			leftOven.translateX(-tableLength/4);
			microwaveTable.add(leftOven);
			
			var rightOven = makeMicrowaveOvenMesh("rightOven");
			rightOven.rotateOnAxis(V.Y_AXIS,-Math.PI/2);
			rightOven.translateX(tableLength/4);
			microwaveTable.add(rightOven);
			
			microwaveTable.rotateOnAxis(V.Y_AXIS,Math.PI/2);
			microwaveTable.translateX((V.ROOM_LENGTH/2)-tableWidth/2);
			microwaveTable.scale.z = 0.75;
			scene.add(microwaveTable);
		}
		
		function makeBackTables(){
			for(var j = 1; j <= 3; j++){
				var backTable = makeTableMesh();
				backTable.translateX((-V.ROOM_WIDTH/2)+(tableWidth/2));
				backTable.translateZ((V.ROOM_LENGTH/2)-(j*tableLength)+(tableLength/2));
				backTable.scale.x = 0.75;
				scene.add(backTable);
			}
		}
		makeComputerTable();
		makeMicrowaveTable();
		makeBackTables();
	}
	
	function makeCouches(){
		var couchWidth = 210;
		var couchHeight = 80;
		var couchDepth = 90;
		var buttHeight = 40;
		var backHeight = couchHeight - buttHeight;
		var backDepth = 20;
		var sideWidth = 20;
		var sideHeight = backHeight*(2/3);
		var sideDepth = couchDepth - backDepth;
		
		function makeCouchMesh(){
			var couch = new THREE.Object3D();
			
			var bottomGeometry = new THREE.BoxGeometry(couchWidth, buttHeight, couchDepth);
			var bottomMesh = new THREE.Mesh(bottomGeometry, couchMaterial);
			bottomMesh.receiveShadow = true;
			couch.add(bottomMesh);
			
			var backGeometry = new THREE.BoxGeometry(couchWidth, backHeight, backDepth);
			var backMesh = new THREE.Mesh(backGeometry, couchMaterial);
            //backMesh.castShadow = true;
			backMesh.translateY(buttHeight);
			backMesh.translateZ((couchDepth-backDepth)/2);
			couch.add(backMesh);
			
			for(var c = 0; c < 2; c++){
				var sideGeometry = new THREE.BoxGeometry(sideWidth, sideHeight, sideDepth);
				var sideMesh = new THREE.Mesh(sideGeometry, couchMaterial);
                //sideMesh.castShadow = true;
				sideMesh.translateY(buttHeight- (sideHeight/2));
				sideMesh.translateZ(-backDepth/2);
				var xOffset = (c == 0) ? (couchWidth-sideWidth)/2 : (sideWidth-couchWidth)/2;
				sideMesh.translateX(xOffset);
				couch.add(sideMesh);
			}
			couch.translateY(buttHeight/2);
			return couch;
		}
		var leftCouch = makeCouchMesh();
		leftCouch.translateZ(-couchDepth/2);
		scene.add(leftCouch);
		
		var rightCouch = makeCouchMesh();
		rightCouch.rotateOnAxis(V.Y_AXIS,Math.PI);
		rightCouch.translateZ(-10-couchDepth/2);
		scene.add(rightCouch);
		
		var thirdCouch = makeCouchMesh();
		thirdCouch.rotateOnAxis(V.Y_AXIS,-Math.PI/2);
		thirdCouch.translateX(couchWidth);
		thirdCouch.translateZ(couchDepth*(3/2));
		thirdCouch.scale.z = 0.8;
		scene.add(thirdCouch);
	}
	
	//TODO use shiny shader?
	function makeLocker(){
		var lockerDimensions = new THREE.Vector3(60,160,60);
		var locker = makeHollowBoxWithDoor(lockerDimensions, 5, "locker", metalMaterial);
		locker.rotateOnAxis(V.Y_AXIS, -Math.PI/2);
		locker.translateZ(-(V.ROOM_WIDTH-60)/2);
		locker.translateX(-50);
		
		var keycard = makeRectangleMesh(10,6,"keycard",keycardMaterial);
		keycard.rotateOnAxis(V.X_AXIS,Math.PI/2);
		keycard.translateZ(77);
		pickable.push(keycard);
		locker.add(keycard);

		scene.add(locker);
	}
	
	var shelfWidth = 100;
	var shelfHeight = 200;
	var shelfDepth = 40; //TODO get measurement
	var shelfThickness = 3;
	function makeShelfMesh(){
		var shelf = new THREE.Object3D();
		
		var backGeometry = new THREE.BoxGeometry(shelfWidth, shelfHeight, shelfThickness);
		var backMesh = new THREE.Mesh(backGeometry, shelfMaterial);
        //backMesh.castShadow = true;
		backMesh.translateZ(-shelfDepth/2);
		shelf.add(backMesh);
		
		for(var i = 0; i < 2; i++){
			var sideGeometry = new THREE.BoxGeometry(shelfThickness, shelfHeight, shelfDepth);
			var sideMesh = new THREE.Mesh(sideGeometry, shelfMaterial);
            //sideMesh.castShadow = true;
			var xOffset = (i == 0) ? shelfWidth/2 : -shelfWidth/2 ;
			sideMesh.translateX(xOffset);
			shelf.add(sideMesh);
		}
		
		for(var i = 0; i < 6; i++){
			var shelfGeometry = new THREE.BoxGeometry(shelfWidth, shelfThickness, shelfDepth);
			var shelfMesh = new THREE.Mesh(shelfGeometry,shelfMaterial);
            //shelfMesh.castShadow = true;
			var yOffset = (i * shelfHeight/5) - (shelfHeight/2);
			shelfMesh.translateY(yOffset);
			shelf.add(shelfMesh);
		}
		
		shelf.translateY(shelfHeight/2);
		return shelf;
	}
	
	function makeShelves(){
		var lostShelf = makeShelfMesh();
		lostShelf.rotateOnAxis(V.Y_AXIS, -Math.PI/2);
		lostShelf.translateZ(-(V.ROOM_WIDTH-shelfDepth)/2);
		lostShelf.translateX(50);
		scene.add(lostShelf);
		highBox = makeSimpleBoxMesh(new THREE.Vector3(50,30,30),"highBox",cardboardMaterial);
		highBox.translateY((shelfHeight+30)/2);
		pickable.push(highBox);
		lostShelf.add(highBox);
		
		var moneyShelf = makeShelfMesh();
		moneyShelf.rotateOnAxis(V.Y_AXIS, Math.PI);
		moneyShelf.translateZ(-(V.ROOM_LENGTH-shelfDepth)/2 +V.BLOCKER_LENGTH);
		moneyShelf.translateX(-(V.ROOM_WIDTH-shelfWidth)/2 + shelfDepth);
		moneyShelf.scale.set(0.9,0.9,0.9);
		moneyShelf.translateY(-10);
		scene.add(moneyShelf);
		
		var cutleryShelf = makeShelfMesh();
		cutleryShelf.translateZ(-(V.ROOM_LENGTH-shelfDepth)/2);
		cutleryShelf.scale.set(0.8,1,1);
		cutleryShelf.translateX(-shelfWidth);
		scene.add(cutleryShelf);
	}
	//TODO add more posters
	function makeRectangles(){
		var whiteboardLength = 100;
		var whiteboard = makeRectangleMesh(whiteboardLength,whiteboardLength,"whiteboard",whiteboardMaterial);
		whiteboard.rotateOnAxis(V.Y_AXIS, -Math.PI/2);
		whiteboard.translateZ(V.ROOM_WIDTH/2);
		whiteboard.translateY(tableHeight + whiteboardLength/2); //Table Height
		whiteboard.translateX((V.ROOM_LENGTH/2)-whiteboardLength);
		pickable.push(whiteboard);
		scene.add(whiteboard);
		
		var windowWidth = 90;//TODO tweak measurements
		var windowHeight = 70; 
		for(var w = 0; w < 4; w++){
			var window = makeRectangleMesh(windowWidth, windowHeight, "window", windowMaterial);
			window.rotateOnAxis(V.Y_AXIS, -Math.PI/2);
			window.translateZ(V.ROOM_WIDTH/2);
			window.translateY(V.ROOM_HEIGHT - windowHeight/2);
			var xOffset = (w > 1) ? (V.ROOM_LENGTH/2)- (windowWidth*(5/4)) : (V.ROOM_LENGTH/2)- (windowWidth*(19/8));
			window.translateX(Math.pow((-1),w) * xOffset);
			pickable.push(window);
			scene.add(window);
          
            var windowLight = new THREE.PointLight(0x38c4d0, V.WINDOW_LIGHT_INTENSITY, 0, 0.3);
            //windowLight.castShadow = true;
           // windowLight.shadowDarkness = 0.2;
            windowLight.translateZ(V.ROOM_HEIGHT);
            windowLight.translateZ(V.ROOM_WIDTH/2);
            windowLight.translateY(V.ROOM_HEIGHT - windowHeight/2);
            windowLight.translateX(Math.pow((-1),w) * xOffset);
            scene.add(windowLight);
		}
	}
	
	function makeTV(){
		var tv = new THREE.Object3D();
		var base = makeShelfMesh();
		base.translateY(-shelfHeight*(4/5));
		tv.add(base);
		
		//TODO use proper texture
		var tvMesh = makeMonitorMesh("tv",textures.tvTexture);
		tvMesh.scale.set(2,3/2,1); //TODO adjust these
		tvMesh.translateY(shelfHeight/5 + 50);//monitor height
		tv.add(tvMesh);
		
		 //TODO this will be removed once picked
		var gunMesh = makeGunMesh();
		gunMesh.rotateOnAxis(V.X_AXIS, -Math.PI/2);
		gunMesh.translateZ(5);
		gunMesh.name = "gunMesh";
		tv.add(gunMesh);
		
		tv.rotateOnAxis(V.Y_AXIS, -Math.PI*(3/4));
		tv.translateZ(-250);
		tv.translateX(100);
		scene.add(tv);
	}
	
	function makeGunMesh(){
		//TODO can only select parts of the gun for now...
		var gun = new THREE.Object3D();
		gun.name = "gun";
		
		var bodyWidth = 30;
		var bodyHeight = 10;
		var bodyDepth = 5;
		var bodyDimensions = new THREE.Vector3(bodyWidth,bodyHeight,bodyDepth);
		var bodyMesh = makeSimpleBoxMesh(bodyDimensions,"gun",bluePlasticMaterial);
		pickable.push(bodyMesh);
		gun.add(bodyMesh);
		
		var gripWidth = 5;
		var gripHeight = 11;
		var gripDimensions = new THREE.Vector3(gripWidth,gripHeight,bodyDepth-1);
		var gripMesh = makeSimpleBoxMesh(gripDimensions,"gun",greyPlasticMaterial);
		gripMesh.translateX((bodyWidth-gripWidth)/2);
		gripMesh.translateY(-gripHeight+2);
		gripMesh.rotateOnAxis(V.Z_AXIS,Math.PI/8);
		pickable.push(gripMesh);
		gun.add(gripMesh);
		
		var chambersGeom = new THREE.CylinderGeometry(4,4,10);
		var chambersMesh = new THREE.Mesh(chambersGeom, orangePlasticMaterial);
        //chambersMesh.castShadow = true;
		chambersMesh.rotateOnAxis(V.Z_AXIS,Math.PI/2);
		chambersMesh.translateY(bodyWidth/6);
		chambersMesh.name = "gun";
		pickable.push(chambersMesh);
		gun.add(chambersMesh);
		
		return gun;
	}

	function makeCameraGun(){
		cameraGun = makeGunMesh();
		cameraGun.name = "cameraGun";
		V.camera.add(cameraGun);
		cameraGun.rotateOnAxis(V.Y_AXIS, -Math.PI/2);
		cameraGun.translateY(-10);
		cameraGun.translateX(30);//Hide it behind camera until gun is found
	}

	function makeBullet(){
		var bulletGeom = new THREE.CylinderGeometry(0.5,0.5,4);
		bullet = new THREE.Mesh(bulletGeom, bulletMaterial);
		bullet.rotateOnAxis(V.X_AXIS, Math.PI/2);
		bullet.translateZ(2);
		bullet.translateY(10);//Hide it behind camera 
		bullet.name = "bullet";
		V.camera.add(bullet);

	}
	
	function makeShinySphere(){
		var shinySphereGeom = new THREE.SphereGeometry(10,32,32);
		var shinySphereMesh = new THREE.Mesh(shinySphereGeom, blackMetalMaterial);
		shinySphereMesh.castShadow = true;
		shinySphereMesh.translateZ(4);
		shinySphereMesh.translateY(88);
		scene.add(shinySphereMesh);
	}
	
	function createEverything(scene) {
		
		//Unpickable objects
		makeCrosshairs();
		makeBullet();
		makeCameraGun();
		makeFloor();
		makeCeiling();
		makeBlocker();
		makeWalls();
		makeLights();
		makeTables();
		makeCouches(); 
		makeShelves();
		makeShinySphere();
		
		//Pickable objects
		makeFridges();
		makeDoor();
		makeCabinets();
		makeLightSwitch();
		makeLocker();
		makeRectangles();
		makeTV();
		
	}
	
    createEverything(scene);

    /*for (var obj in objects) {
      scene.add(objects[obj]);
    }*/

  }

  //TODO get rotation animation working properly
  function toggleDoor(boxName){
		var doorPanel = scene.getObjectByName(boxName);
		var doorParam = doorPanel.geometry.parameters;
        game.animateDoor = true;
        game.doorToBeAnimated = doorPanel;	
		game.doorFramesLeft = 12;
	}

  function switchLights(){
      for (var a = 0; a < 1; a++) {
        if(ceilingLights[a].intensity == 0){
            ceilingLights[a].intensity = V.CEILING_LIGHT_INTENSITY;
        }
        else {
            ceilingLights[a].intensity = 0;
        }
      }
      if (centerLight.intensity == 0) {
        centerLight.intensity = V.CENTER_LIGHT_INTENSITY;
      } else {
        centerLight.intensity = 0;
      }

  }
  
  function hideObjectByName(objName){
	  scene.getObjectByName(objName).translateX(-1000);
	  scene.getObjectByName(objName).translateY(-1000);
	  scene.getObjectByName(objName).translateZ(-1000);
  }
  
  function equipGun(){
	  //TODO
	  cameraGun.translateX(-60);
	game.gunEquipped = true;
  }

  function moveBullet(dist){
	//TODO use stuff from contorl
	bullet.translateY(-dist);
	
  }
  
  //TODO just save local variables?
  function switchComputerScreen(screenName){
	var texture;
	switch(screenName){
		case("loggedOn"):
			texture = textures.loggedOnTexture;
		break;
		case("bsod"):
			texture = textures.bsodTexture;
		break;
	}
	for(var m = 0; m < 2; m++){
		var monitorName = "monitor" + m.toString();
		scene.getObjectByName(monitorName).material.map = texture;
	}
  }
  //TODO
  function dropHighBox(){
	  highBox.translateY(-200);
	  highBox.translateZ(50)
  }

  
  function getObjectPicked() {
    return objectPicker.getPickedObject();
  }
 
  
  return {
    canvas: canvas,
    scene: scene,
    renderer: renderer,
	pickable: pickable,
    fpCamera: fpCamera,
	objectPicker: objectPicker,
	switchLights: switchLights,
	toggleDoor:toggleDoor,
	hideObjectByName:hideObjectByName,
	equipGun:equipGun,
	moveBullet:moveBullet,
	switchComputerScreen:switchComputerScreen,
	dropHighBox:dropHighBox,
	getObjectPicked:getObjectPicked,
    setup: function () {
      initRenderer();
      initResize();
      initObjects();
      fpCamera.initCamera();
      initLockMouse();
	  switchLights();
    }
  };
  
})(V);
